close all; clear; clc;
addpath(genpath('.\utils'))

%
% DEPcode (2018):
% This code implements a version of the descriptor described in the paper:
%
% Discrete time Evolution Process Descriptor for shape analysis and matching
% ACM Transactions on Graphics, Vol. 37, No. 1, Article 4. January 2018
%
% Simone Melzi - University of Verona
% Maks Ovsjanikov - LIX cole Polytechnique 
% Giorgio Roffo - University of Glasgow
% Marco Cristani - University of Verona
% Umberto Castellani - University of Verona
%
% use DEP descriptor in the functional maps framework similarly to 
% figures 9,10 in the paper
%
% original parameters;
% c = 0.1; flag_dist = 'Dijkstra'; n_scale = 6; f0 = ones(Src.nv,1);
% Src.n_points = Src.nv; Tar.n_points = Tar.nv;
%
% If you set flag_dist = 'SpectroMeter' it is 10 times faster.
% If you set Src.n_points < Src.nv and Tar.n_points < Tar.nv
% the computation is faster.
%

%% load shapes
[X,T] = readOff('.\data\tr_reg_089');
Src = MeshInfo(X, T, 200);

[X,T] = readOff('.\data\tr_reg_090');
Tar = MeshInfo(X, T, 200);

%% select parameters
% DEP parameters
n_scale = 6; % 100; % 16;
c = 0.1; % 0.5; % 0.9;
flag_dist = 'SpectroMeter'; % 'Dijkstra'; % SpectroMeter is 10 times faster
Src.n_points = Src.nv; % 3000; 
Tar.n_points = Tar.nv; % 3000; 
if n_scale == 16
    deltas = [0.01,0.02,0.025,0.03,0.04 0.05,0.07,0.075,0.08, 0.09,0.1,...
              0.15, 0.35, 0.55, 0.8, 1];
elseif n_scale == 100
    deltas = [0.005:0.005:0.375,0.4:0.025:1];
elseif n_scale == 6
    deltas100 = [0.005:0.005:0.375,0.4:0.025:1];
    deltas_idxs = [1,20:20:100]; %[1 20 40 60 80 100];
    deltas = deltas100(deltas_idxs);
end

% fmaps parameters
numEigsSrc = 60;
numEigsTar = 60;
sym = 1;
flag_landmarks = 0;

%% prepare landmarks and descriptors
l1 = [1000 6000]';
l2 = l1;
tmpSrc = waveKernelMap(Src.laplaceBasis, Src.eigenvalues, Src.Ae, 200, l1);
tmpTar = waveKernelMap(Tar.laplaceBasis, Tar.eigenvalues, Tar.Ae, 200, l2);
Src.landmarks = tmpSrc(:,1:39:200); clear tmpSrc;
Tar.landmarks = tmpTar(:,1:39:200); clear tmpTar;

% WKS
SrcWKS = waveKernelSignature(Src.laplaceBasis, Src.eigenvalues, Src.Ae, n_scale);
TarWKS = waveKernelSignature(Tar.laplaceBasis, Tar.eigenvalues, Tar.Ae, n_scale);
Src.WKS = SrcWKS; %(:,deltas_idxs);
Tar.WKS = TarWKS; %(:,deltas_idxs);

% DEP
Src = all_in_DEP(Src,c,deltas,flag_dist,Src.n_points);
Tar = all_in_DEP(Tar,c,deltas,flag_dist,Tar.n_points);

%% compute the fmaps
C_gt = Tar.laplaceBasis(:,1:numEigsTar)'*Tar.Ae*Src.laplaceBasis(:,1:numEigsSrc);
if flag_landmarks
    C_WKS = compute_map_desc(Src,Tar,[Src.WKS, Src.landmarks],[Tar.WKS, Tar.landmarks],numEigsSrc);
    C_DEP = compute_map_desc(Src,Tar,[Src.DEP, Src.landmarks],[Tar.DEP, Tar.landmarks],numEigsSrc);    
else
    C_WKS = compute_map_desc(Src,Tar,Src.WKS,Tar.WKS,numEigsSrc);
    C_DEP = compute_map_desc(Src,Tar,Src.DEP,Tar.DEP,numEigsSrc);
end

%% compute the p2p maps

p2p_gt = annquery(C_gt*(Src.laplaceBasis(:,1:numEigsSrc)'),Tar.laplaceBasis(:,1:numEigsTar)',1);
p2p_WKS = annquery(C_WKS*(Src.laplaceBasis(:,1:numEigsSrc)'),Tar.laplaceBasis(:,1:numEigsTar)',1);
p2p_DEP = annquery(C_DEP*(Src.laplaceBasis(:,1:numEigsSrc)'),Tar.laplaceBasis(:,1:numEigsTar)',1);
% p2p_gt = knnsearch(Tar.laplaceBasis(:,1:numEigsTar),( C_gt*(Src.laplaceBasis(:,1:numEigsSrc)'))');
% p2p_WKS = knnsearch(Tar.laplaceBasis(:,1:numEigsTar),( C_WKS*(Src.laplaceBasis(:,1:numEigsSrc)'))');
% p2p_DEP = knnsearch(Tar.laplaceBasis(:,1:numEigsTar),( C_DEP*(Src.laplaceBasis(:,1:numEigsSrc)'))');

%%
max_th = 0.3; thresh = 0:0.001:1;
D = Tar.dists; diam = max(D(:));
idx = find(thresh==max_th);
errs = zeros(Src.nv,3);
for i=1:Src.nv
    if sym == 1
        tmp = load(['.\data\sym_labels_FAUST.mat']); 
        sym = tmp.idxs; clear tmp;
        errs(i,1) = min(D(i,p2p_gt(i)),D(sym(i),p2p_gt(i)));
        errs(i,2) = min(D(i,p2p_WKS(i)),D(sym(i),p2p_WKS(i)));
        errs(i,3) = min(D(i,p2p_DEP(i)),D(sym(i),p2p_DEP(i)));

    else
        errs(i,1) = D(i,p2p_gt(i));
        errs(i,2) = D(i,p2p_WKS(i));
        errs(i,3) = D(i,p2p_DEP(i));
    end
end
errs = errs ./ diam;
curve_gt = calc_err_curve(errs(:,1), thresh);
curve_WKS = calc_err_curve(errs(:,2), thresh);
curve_DEP = calc_err_curve(errs(:,3), thresh);
AUCs(1) = trapz(thresh(1:idx), curve_gt(1:idx))/max_th; 
AUCs(2) = trapz(thresh(1:idx), curve_WKS(1:idx))/max_th; 
AUCs(3) = trapz(thresh(1:idx), curve_DEP(1:idx))/max_th; 

figure
plot(thresh, curve_gt, 'linewidth', 2); hold on;
plot(thresh, curve_WKS, 'linewidth', 2);
plot(thresh, curve_DEP, 'linewidth', 2);
set(gca, 'xlim', [0 max_th])
set(gca, 'ylim', [0 100])
set(gca, 'xgrid', 'on')
set(gca, 'ygrid', 'on')
xlabel('Geodesic error')
ylabel('% Correspondences')
title('p2p evaluation')
legend(['gt ', num2str(AUCs(1))],['WKS ', num2str(AUCs(2))],['DEP ', num2str(AUCs(3))],'Location','southeast');
