close all; clear; clc;
addpath(genpath('.\utils'))

%
% DEPcode (2018):
% This code implements a version of the descriptor described in the paper:
%
% Discrete time Evolution Process Descriptor for shape analysis and matching
% ACM Transactions on Graphics, Vol. 37, No. 1, Article 4. January 2018
%
% Simone Melzi - University of Verona
% Maks Ovsjanikov - LIX cole Polytechnique 
% Giorgio Roffo - University of Glasgow
% Marco Cristani - University of Verona
% Umberto Castellani - University of Verona
%
% produce matching using only descriptors similarly to figures 7,8 
% in the paper
%
% original parameters;
% c = 0.1; flag_dist = 'Dijkstra'; n_scale = 100; f0 = ones(Src.nv,1);
% Src.n_points = Src.nv; Tar.n_points = Tar.nv;
%
% If you set flag_dist = 'SpectroMeter' it is 10 times faster.
% If you set Src.n_points < Src.nv and Tar.n_points < Tar.nv
% the computation is faster.
%

%% load shapes
[X,T] = readOff('.\data\tr_reg_089');
Src = MeshInfo(X, T, 200);

[X,T] = readOff('.\data\tr_reg_090');
Tar = MeshInfo(X, T, 200);

%% select parameters
% DEP parameters
n_scale = 16; % 6; % 100;
c = 0.1; % 0.5; % 0.9;
flag_dist = 'SpectroMeter'; % 'Dijkstra'; % SpectroMeter is 10 times faster
Src.n_points = Src.nv; % 3000;
Tar.n_points = Tar.nv; % 3000;
if n_scale == 16
    deltas = [0.01,0.02,0.025,0.03,0.04 0.05,0.07,0.075,0.08, 0.09,0.1,...
              0.15, 0.35, 0.55, 0.8, 1];
elseif n_scale == 100
    deltas = [0.005:0.005:0.375,0.4:0.025:1];
elseif n_scale == 6
    deltas100 = [0.005:0.005:0.375,0.4:0.025:1];
    deltas_idxs = [1,20:20:100]; %[1 20 40 60 80 100];
    deltas = deltas100(deltas_idxs);
end

%% prepare descriptors
% WKS
Src.WKS = waveKernelSignature(Src.laplaceBasis, Src.eigenvalues, Src.Ae, 200); % n_scale);
Tar.WKS = waveKernelSignature(Tar.laplaceBasis, Tar.eigenvalues, Tar.Ae, 200); %n_scale);

Src = all_in_DEP(Src,c,deltas,flag_dist,Src.n_points);
Tar = all_in_DEP(Tar,c,deltas,flag_dist,Tar.n_points);

%% compute the p2p maps
p2p_WKS = knnsearch(Tar.WKS,Src.WKS);
p2p_DEP = knnsearch(Tar.DEP,Src.DEP);

%%
max_th = 0.25; thresh = 0:0.001:1;
D = Tar.dists; diam = max(D(:));
idx = find(thresh==max_th);
errs = zeros(Src.nv,2);

for i=1:Src.nv
    errs(i,1) = D(i,p2p_WKS(i));
    errs(i,2) = D(i,p2p_DEP(i));
end
errs = errs ./ diam;
curve_WKS = calc_err_curve(errs(:,1), thresh);
curve_DEP = calc_err_curve(errs(:,2), thresh);
AUCs(1) = trapz(thresh(1:idx), curve_WKS(1:idx))/max_th; 
AUCs(2) = trapz(thresh(1:idx), curve_DEP(1:idx))/max_th; 

figure
plot(thresh, curve_WKS, 'linewidth', 2); hold on;
plot(thresh, curve_DEP, 'linewidth', 2);
set(gca, 'xlim', [0 max_th])
set(gca, 'ylim', [0 100])
set(gca, 'xgrid', 'on')
set(gca, 'ygrid', 'on')
xlabel('Geodesic error')
ylabel('% Correspondences')
title('p2p evaluation')
legend(['WKS200 ', num2str(AUCs(1))],['DEP',num2str(n_scale),' ', num2str(AUCs(2))],'Location','southeast');
